@@ -103,10 +103,12 @@ class ParseFileFlags:
             help='Overrides to print the Z3 constraints or else the unsat_core constraints if there is no model.'
         )
         parser.add_argument(
-            '--save_output',
+            '--save',
             '-s',
-            action='store_true',
-            help='Overrides to prompt user to save output.'
+            nargs='*',  # Allow 0 or more arguments
+            choices=['markdown', 'json', 'jupyter'],
+            default=None,  # When flag not used at all
+            help='Save output in specified formats. Without arguments: all formats. With arguments: only specified formats (e.g., --save jupyter markdown)'
         )
         parser.add_argument(
             '--print_impossible',
@@ -140,22 +142,10 @@ class ParseFileFlags:
             help='Overrides to display world histories vertically with time flowing from top to bottom.'
         )
         parser.add_argument(
-            '--output-mode',
-            choices=['batch', 'sequential'],
-            default='batch',
-            help='Output mode when saving: batch (single file) or sequential (multiple files)'
-        )
-        parser.add_argument(
-            '--sequential-files',
-            choices=['single', 'multiple'],
-            default='multiple',
-            help='For sequential mode: single file or multiple files'
-        )
-        parser.add_argument(
-            '--interactive',
-            '-I',
+            '--sequential',
+            '-q',
             action='store_true',
-            help='Enable interactive save mode - prompts after each model to save'
+            help='Save models sequentially with prompts'
         )
         return parser
 
@@ -179,14 +169,13 @@ class ParseFileFlags:
             'l': 'load_theory',
             'm': 'maximize',
             'n': 'non_null',
-            'p': 'print_constraints',
-            's': 'save_output',
+            'q': 'sequential',
+            's': 'save',
             'i': 'print_impossible',
             'v': 'version',
             'u': 'upgrade',
             'z': 'print_z3',
-            'a': 'align_vertically',
-            'I': 'interactive'
+            'a': 'align_vertically'
         }
         
         # Store the original command line arguments
@@ -225,7 +214,7 @@ def main():
     No parameters or return values as this is the main program entry point.
     """
     # Check for Jupyter dependencies if they'll be needed
-    jupyter_flags = ["-j", "--jupyter", "-n", "--notebook"]
+    jupyter_flags = ["-j", "--jupyter"]
     needs_jupyter = any(flag in sys.argv for flag in jupyter_flags)
     
     if needs_jupyter:
@@ -261,15 +250,16 @@ def main():
         builder = BuildProject(semantic_theory_name)
         builder.ask_generate()
         return
-
+    
     module = BuildModule(module_flags)
 
     # TODO: create print/save class
     if module.general_settings["maximize"]:
-        module.run_comparison()
+        module.comparison.run_comparison()
         return
 
-    module.run_examples()
+    # Run examples for console output and documentation capture
+    module.runner.run_examples()
 
 def run():
     """Entry point for the application."""
