@@ -8,6 +8,12 @@ import sys
 import time
 from contextlib import redirect_stdout
 from string import Template
+from typing import Dict, List, Tuple, Optional, Any, TextIO, Union, TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from model_checker.syntactic import Sentence
+    from .constraints import ModelConstraints
+    import z3
 
 
 class ModelDefaults:
@@ -38,7 +44,7 @@ class ModelDefaults:
         main_point (dict): The primary evaluation point for the model
     """
 
-    def __init__(self, model_constraints, settings):
+    def __init__(self, model_constraints: 'ModelConstraints', settings: Dict[str, Any]) -> None:
         # Define ANSI color codes
         self.COLORS = {
             "default": "\033[37m",  # WHITE
@@ -89,7 +95,8 @@ class ModelDefaults:
         
         # Reset Z3 context before creating a new solver to ensure isolation
         from model_checker.utils import Z3ContextManager
-        Z3ContextManager.reset_context()
+        from ..utils.context import reset_z3_context
+        reset_z3_context()
 
         # Solve Z3 constraints and store results
         solver_results = self.solve(self.model_constraints, self.max_time)
@@ -99,7 +106,7 @@ class ModelDefaults:
         if self.timeout or self.z3_model is None:
             return
 
-    def _process_solver_results(self, solver_results):
+    def _process_solver_results(self, solver_results: Tuple[bool, Any, bool, float]) -> None:
         """Process and store the results from the Z3 solver.
         
         Takes the raw solver results and updates the model's state attributes including
@@ -127,7 +134,7 @@ class ModelDefaults:
         else:
             self.unsat_core = z3_model
 
-    def _setup_solver(self, model_constraints):
+    def _setup_solver(self, model_constraints: 'ModelConstraints') -> 'z3.Solver':
         """Initialize Z3 solver and add all model constraints with tracking labels.
         
         Sets up a new Z3 solver instance and adds all constraints from the model_constraints
@@ -166,7 +173,7 @@ class ModelDefaults:
                 
         return solver
 
-    def _create_result(self, is_timeout, model_or_core, is_satisfiable, start_time):
+    def _create_result(self, is_timeout: bool, model_or_core: Any, is_satisfiable: bool, start_time: float) -> Tuple[bool, Any, bool, float]:
         """Creates a standardized tuple containing solver results with runtime.
         
         Args:
@@ -182,7 +189,7 @@ class ModelDefaults:
         runtime = round(time.time() - start_time, 4)
         return is_timeout, model_or_core, is_satisfiable, runtime
 
-    def _cleanup_solver_resources(self):
+    def _cleanup_solver_resources(self) -> None:
         """Explicitly clean up Z3 resources to ensure complete isolation between examples.
         
         This method is crucial for preventing state leakage between different examples.
@@ -202,7 +209,7 @@ class ModelDefaults:
         if hasattr(self, 'example_context'):
             self.example_context = None
     
-    def solve(self, model_constraints, max_time):
+    def solve(self, model_constraints: 'ModelConstraints', max_time: int) -> Tuple[bool, Any, bool, Optional[float]]:
         """Uses the Z3 solver to find a model satisfying the given constraints.
         
         Creates a completely fresh Z3 context for each example to ensure
@@ -246,13 +253,13 @@ class ModelDefaults:
             return self._create_result(False, self.solver.unsat_core(), False, start_time)
             
         except RuntimeError as e:
-            print(f"An error occurred during solving: {e}")
-            return True, None, False, None
+            from .errors import ModelSolverError
+            raise ModelSolverError(f"Z3 solver encountered an error: {e}") from e
         finally:
             # Ensure proper cleanup to prevent any possible state leakage
             self._cleanup_solver_resources()
     
-    def re_solve(self):
+    def re_solve(self) -> Tuple[bool, Any, bool, float]:
         """Re-solve the existing model constraints with the current solver state.
         
         Attempts to find a new solution using the existing solver instance and its
@@ -286,10 +293,10 @@ class ModelDefaults:
             return self._create_result(False, self.solver.unsat_core(), False, start_time)
             
         except RuntimeError as e:
-            print(f"An error occurred while running `re_solve()`: {e}")
-            return True, None, False, None
+            from .errors import ModelSolverError
+            raise ModelSolverError(f"Re-solve operation failed: {e}") from e
 
-    def check_result(self):
+    def check_result(self) -> Optional[bool]:
         """Checks if the model's result matches the expected outcome.
         
         Compares the actual model status (satisfiable/unsatisfiable) against the
@@ -304,7 +311,7 @@ class ModelDefaults:
             return None
         return self.z3_model_status == self.settings["expectation"]
 
-    def interpret(self, sentences):
+    def interpret(self, sentences: List['Sentence']) -> None:
         """Recursively updates sentences with their semantic interpretations in the model.
         
         For each sentence in the input list, creates and attaches a proposition object
@@ -327,7 +334,7 @@ class ModelDefaults:
                 self.interpret(sent_obj.arguments)
             sent_obj.update_proposition(self)
 
-    def print_grouped_constraints(self, output=sys.__stdout__):
+    def print_grouped_constraints(self, output: TextIO = sys.__stdout__) -> None:
         """Prints all model constraints organized by their logical category.
         
         Displays constraints grouped into four categories:
@@ -346,25 +353,39 @@ class ModelDefaults:
         Args:
             output (file, optional): Output stream to write to. Defaults to sys.stdout.
         """
-        groups = {
-            "FRAME": [],
-            "MODEL": [],
-            "PREMISES": [],
-            "CONCLUSIONS": []
-        }
+        # Get constraints based on model status
+        constraints = self._get_relevant_constraints(output)
         
-        # Get the relevant constraints based on model status
+        # Print summary counts
+        self._print_constraint_summary(constraints, output)
+        
+        # Organize and print constraint groups
+        groups = self._organize_constraint_groups(constraints)
+        self._print_constraint_groups(groups, output)
+    
+    def _get_relevant_constraints(self, output: TextIO) -> List[Any]:
+        """Get the relevant constraints based on model satisfiability status.
+        
+        Returns:
+            list: Constraints to display (all constraints if SAT, unsat core if UNSAT)
+        """
         if self.z3_model:
             print("SATISFIABLE CONSTRAINTS:", file=output)
-            constraints = self.model_constraints.all_constraints
+            return self.model_constraints.all_constraints
         elif self.unsat_core is not None:
             print("UNSATISFIABLE CORE CONSTRAINTS:", file=output)
-            constraints = [self.constraint_dict[str(c)] for c in self.unsat_core]
+            return [self.constraint_dict[str(c)] for c in self.unsat_core]
         else:
             print("NO CONSTRAINTS AVAILABLE", file=output)
-            constraints = []
-            
-        # Print summary of constraint groups
+            return []
+    
+    def _print_constraint_summary(self, constraints: List[Any], output: TextIO) -> None:
+        """Print summary counts for each constraint category.
+        
+        Args:
+            constraints: List of constraints to summarize
+            output: Output stream to write to
+        """
         frame_count = sum(1 for c in constraints if c in self.model_constraints.frame_constraints)
         model_count = sum(1 for c in constraints if c in self.model_constraints.model_constraints) 
         premise_count = sum(1 for c in constraints if c in self.model_constraints.premise_constraints)
@@ -374,8 +395,23 @@ class ModelDefaults:
         print(f"- Model constraints: {model_count}", file=output)
         print(f"- Premise constraints: {premise_count}", file=output)
         print(f"- Conclusion constraints: {conclusion_count}\n", file=output)
+    
+    def _organize_constraint_groups(self, constraints: List[Any]) -> Dict[str, List[Any]]:
+        """Organize constraints into logical groups.
+        
+        Args:
+            constraints: List of constraints to organize
+            
+        Returns:
+            dict: Constraints organized by group (FRAME, MODEL, PREMISES, CONCLUSIONS)
+        """
+        groups = {
+            "FRAME": [],
+            "MODEL": [],
+            "PREMISES": [],
+            "CONCLUSIONS": []
+        }
         
-        # Organize constraints into groups
         for constraint in constraints:
             if constraint in self.model_constraints.frame_constraints:
                 groups["FRAME"].append(constraint)
@@ -386,14 +422,22 @@ class ModelDefaults:
             elif constraint in self.model_constraints.conclusion_constraints:
                 groups["CONCLUSIONS"].append(constraint)
         
-        # Print each group
+        return groups
+    
+    def _print_constraint_groups(self, groups: Dict[str, List[Any]], output: TextIO) -> None:
+        """Print each group of constraints with headers and numbering.
+        
+        Args:
+            groups: Dictionary of constraint groups
+            output: Output stream to write to
+        """
         for group_name, group_constraints in groups.items():
             if group_constraints:  # Only print groups that have constraints
                 print(f"{group_name} CONSTRAINTS:", file=output)
                 for index, con in enumerate(group_constraints, start=1):
                     print(f"{index}. {con}\n", file=output)
 
-    def print_constraints(self, constraints, name, output=sys.__stdout__):
+    def print_constraints(self, constraints: List[Any], name: str, output: TextIO = sys.__stdout__) -> None:
         """Prints a numbered list of constraints with appropriate header.
         
         Args:
@@ -418,7 +462,7 @@ class ModelDefaults:
         for index, con in enumerate(constraints, start=1):
             print(f"{index}. {con}\n", file=output)
 
-    def build_test_file(self, output):
+    def build_test_file(self, output: TextIO) -> None:
         """Generates a test file from the current model configuration and results.
         
         Creates a Python test file containing the model settings, premises, conclusions,
@@ -484,7 +528,7 @@ class ModelDefaults:
         inputs_content = inputs_template.substitute(inputs_data)
         print(inputs_content, file=output)
 
-    def recursive_print(self, sentence, eval_point, indent_num, use_colors):
+    def recursive_print(self, sentence: 'Sentence', eval_point: Dict[str, Any], indent_num: int, use_colors: bool) -> None:
         """Recursively prints a sentence and its subformulas with their truth values.
 
         This method handles both atomic and complex sentences, printing them with
@@ -512,7 +556,7 @@ class ModelDefaults:
         operator = sentence.original_operator
         operator.print_method(sentence, eval_point, indent_num, use_colors)  # Print complex sentence
 
-    def print_input_sentences(self, output):
+    def print_input_sentences(self, output: TextIO) -> None:
         """Prints the premises and conclusions with their semantic interpretations.
         
         For each premise and conclusion, recursively prints the sentence along with
@@ -528,44 +572,56 @@ class ModelDefaults:
             - Premises are numbered starting from 1
             - Conclusions continue the numbering after premises
         """
-        from contextlib import redirect_stdout
-        
-        def print_sentences(title_singular, title_plural, sentences, start_index, destination):
-            """Helper function to print a list of sentences with a title."""
-            if not sentences:
-                return
-                
-            if not self.z3_model:
-                print("No valid model available - cannot interpret sentences", file=output)
-                return
-                
-            title = title_singular if len(sentences) < 2 else title_plural
-            print(title, file=output)
-            for index, sentence in enumerate(sentences, start=start_index):
-                print(f"{index}.", end="", file=output)
-                with redirect_stdout(destination):
-                    use_colors = output is sys.__stdout__
-                    self.recursive_print(sentence, self.main_point, 1, use_colors)
-                    print(file=output)
-        
+        if not self.z3_model:
+            print("No valid model available - cannot interpret sentences", file=output)
+            return
+            
+        # Print premises
         start_index = 1
-        print_sentences(
+        self._print_sentence_group(
             "INTERPRETED PREMISE:\n",
             "INTERPRETED PREMISES:\n",
             self.premises,
             start_index,
             output
         )
+        
+        # Print conclusions  
         continue_index = len(self.premises) + 1
-        print_sentences(
+        self._print_sentence_group(
             "INTERPRETED CONCLUSION:\n",
             "INTERPRETED CONCLUSIONS:\n",
             self.conclusions,
             continue_index,
             output
         )
+    
+    def _print_sentence_group(self, title_singular: str, title_plural: str, sentences: List['Sentence'], start_index: int, output: TextIO) -> None:
+        """Print a group of sentences with appropriate title and numbering.
+        
+        Args:
+            title_singular: Title to use if there's only one sentence
+            title_plural: Title to use if there are multiple sentences
+            sentences: List of sentences to print
+            start_index: Starting number for the sentences
+            output: Output stream to write to
+        """
+        from contextlib import redirect_stdout
+        
+        if not sentences:
+            return
+            
+        title = title_singular if len(sentences) < 2 else title_plural
+        print(title, file=output)
+        
+        for index, sentence in enumerate(sentences, start=start_index):
+            print(f"{index}.", end="", file=output)
+            with redirect_stdout(output):
+                use_colors = output is sys.__stdout__
+                self.recursive_print(sentence, self.main_point, 1, use_colors)
+                print(file=output)
 
-    def print_model(self, output):
+    def print_model(self, output: TextIO) -> None:
         """Prints the raw Z3 model or unsat core if print_z3 setting is enabled.
         
         This method prints either the complete Z3 model (if constraints are satisfiable)
@@ -586,7 +642,7 @@ class ModelDefaults:
             else:
                 print(self.unsat_core, file=output)
                 
-    def calculate_model_differences(self, previous_structure):
+    def calculate_model_differences(self, previous_structure: 'ModelDefaults') -> Optional[Dict[str, Any]]:
         """Calculate theory-specific differences between this model and a previous one.
         
         This method identifies semantic differences that are meaningful in this theory's
@@ -605,7 +661,7 @@ class ModelDefaults:
         # Default implementation returns None, meaning use basic difference detection
         return None
         
-    def print_model_differences(self, output=sys.stdout):
+    def print_model_differences(self, output: TextIO = sys.stdout) -> None:
         """Prints differences between this model and the previous one.
         
         Default implementation that provides basic difference display.
@@ -619,59 +675,75 @@ class ModelDefaults:
         
         print("\n=== DIFFERENCES FROM PREVIOUS MODEL ===\n", file=output)
         
-        # Print sentence letter differences
-        letter_diffs = self.model_differences.get('sentence_letters', {})
-        if letter_diffs:
-            print("Sentence Letter Changes:", file=output)
-            for letter, values in letter_diffs.items():
-                try:
-                    if 'old' in values and 'new' in values:
-                        print(f"  {letter}: {values['old']} → {values['new']}", file=output)
-                    else:
-                        print(f"  {letter}: changed from previous model", file=output)
-                except (KeyError, TypeError) as e:
-                    print(f"  {letter}: changed from previous model", file=output)
-            print("", file=output)
-            
-        # Print semantic function differences
-        func_diffs = self.model_differences.get('semantic_functions', {})
-        if func_diffs:
-            print("Semantic Function Changes:", file=output)
-            for func_name, values in func_diffs.items():
-                print(f"  {func_name}:", file=output)
-                for input_val, change in values.items():
-                    try:
-                        if 'old' in change and 'new' in change:
-                            print(f"    Input {input_val}: {change['old']} → {change['new']}", file=output)
-                        else:
-                            print(f"    Input {input_val}: changed from previous model", file=output)
-                    except (KeyError, TypeError) as e:
-                        print(f"    Input {input_val}: changed from previous model", file=output)
-            print("", file=output)
-            
-        # Print model structure differences
-        struct_diffs = self.model_differences.get('model_structure', {})
-        if struct_diffs:
-            print("Model Structure Changes:", file=output)
-            for component, values in struct_diffs.items():
-                try:
-                    if 'old' in values and 'new' in values:
-                        print(f"  {component}: {values['old']} → {values['new']}", file=output)
-                    else:
-                        print(f"  {component}: changed from previous model", file=output)
-                except (KeyError, TypeError) as e:
-                    print(f"  {component}: changed from previous model", file=output)
-            print("", file=output)
+        # Print each category of differences
+        self._print_sentence_letter_differences(output)
+        self._print_semantic_function_differences(output)
+        self._print_model_structure_differences(output)
+        self._print_structural_metrics(output)
         
-        # Print structural metrics if available
-        print("Structural Properties:", file=output)
-        print(f"  Worlds: {len(getattr(self, 'z3_world_states', []))}", file=output)
-            
-        # If the model is marked as isomorphic to a previous model, note that
+        # Note if model is isomorphic
         if hasattr(self, 'isomorphic_to_previous') and self.isomorphic_to_previous:
             print("NOTE: This model may be isomorphic to a previous model despite syntactic differences.", file=output)
+    
+    def _print_sentence_letter_differences(self, output: TextIO) -> None:
+        """Print differences in sentence letter values."""
+        letter_diffs = self.model_differences.get('sentence_letters', {})
+        if not letter_diffs:
+            return
+            
+        print("Sentence Letter Changes:", file=output)
+        for letter, values in letter_diffs.items():
+            try:
+                if 'old' in values and 'new' in values:
+                    print(f"  {letter}: {values['old']} → {values['new']}", file=output)
+                else:
+                    print(f"  {letter}: changed from previous model", file=output)
+            except (KeyError, TypeError):
+                print(f"  {letter}: changed from previous model", file=output)
+        print("", file=output)
+    
+    def _print_semantic_function_differences(self, output: TextIO) -> None:
+        """Print differences in semantic functions."""
+        func_diffs = self.model_differences.get('semantic_functions', {})
+        if not func_diffs:
+            return
+            
+        print("Semantic Function Changes:", file=output)
+        for func_name, values in func_diffs.items():
+            print(f"  {func_name}:", file=output)
+            for input_val, change in values.items():
+                try:
+                    if 'old' in change and 'new' in change:
+                        print(f"    Input {input_val}: {change['old']} → {change['new']}", file=output)
+                    else:
+                        print(f"    Input {input_val}: changed from previous model", file=output)
+                except (KeyError, TypeError):
+                    print(f"    Input {input_val}: changed from previous model", file=output)
+        print("", file=output)
+    
+    def _print_model_structure_differences(self, output: TextIO) -> None:
+        """Print differences in model structure."""
+        struct_diffs = self.model_differences.get('model_structure', {})
+        if not struct_diffs:
+            return
+            
+        print("Model Structure Changes:", file=output)
+        for component, values in struct_diffs.items():
+            try:
+                if 'old' in values and 'new' in values:
+                    print(f"  {component}: {values['old']} → {values['new']}", file=output)
+                else:
+                    print(f"  {component}: changed from previous model", file=output)
+            except (KeyError, TypeError):
+                print(f"  {component}: changed from previous model", file=output)
+        print("", file=output)
+    
+    def _print_structural_metrics(self, output: TextIO) -> None:
+        """Print structural properties of the model."""
+        print("Structural Properties:", file=output)
+        print(f"  Worlds: {len(getattr(self, 'z3_world_states', []))}", file=output)
 
-    def print_info(self, model_status, default_settings, example_name, theory_name, output):
+    def print_info(self, model_status: bool, default_settings: Dict[str, Any], example_name: str, theory_name: str, output: TextIO) -> None:
         """Print comprehensive model information and analysis results.
         
         Displays a formatted output containing the model checking results, including
@@ -706,17 +778,17 @@ class ModelDefaults:
         self.model_constraints.print_enumerate(output)
         self._print_runtime_footer(output)
 
-    def _print_section_header(self, example_name, header, output):
+    def _print_section_header(self, example_name: str, header: str, output: TextIO) -> None:
         """Print the section header with example name and result."""
         print(f"{'='*40}", file=output)
         print(f"\nEXAMPLE {example_name}: {header}\n", file=output)
 
-    def _print_model_details(self, theory_name, output):
+    def _print_model_details(self, theory_name: str, output: TextIO) -> None:
         """Print model details including atomic states and semantic theory."""
         print(f"Atomic States: {self.N}\n", file=output)
         print(f"Semantic Theory: {theory_name}\n", file=output)
 
-    def _print_modified_settings(self, default_settings, output):
+    def _print_modified_settings(self, default_settings: Dict[str, Any], output: TextIO) -> None:
         """Print settings that have been modified from their default values.
         
         Compares the current settings against the default configuration and prints
@@ -743,12 +815,34 @@ class ModelDefaults:
                 print(f"  {key} = {value}", file=output)
             print()
 
-    def _print_runtime_footer(self, output):
+    def _print_runtime_footer(self, output: TextIO) -> None:
         """Print Z3 runtime and separator footer."""
         print(f"\nZ3 Run Time: {self.z3_model_runtime} seconds", file=output)
         print(f"\n{'='*40}", file=output)
     
-    def extract_verify_falsify_state(self):
+    def print_all(self, output: TextIO = sys.stdout) -> None:
+        """Print complete model information.
+        
+        This method provides comprehensive output of the model checking results,
+        including interpreted sentences and grouped constraints. It's the primary
+        method for displaying full model details to users.
+        
+        Args:
+            output (file, optional): Output stream to write to. Defaults to sys.stdout.
+        """
+        # Print interpreted sentences (premises and conclusions)
+        self.print_input_sentences(output)
+        
+        # Add separator between sentences and constraints
+        print(file=output)
+        
+        # Print grouped constraints
+        self.print_grouped_constraints(output)
+        
+        # Print raw Z3 model if requested
+        self.print_model(output)
+    
+    def extract_verify_falsify_state(self) -> Dict[Tuple[int, str], Tuple[bool, bool]]:
         """Extract current verify/falsify function values from Z3 model.
         
         This method extracts the current values of verify and falsify functions
@@ -759,10 +853,14 @@ class ModelDefaults:
             dict: Mapping of (state, letter) -> (verify_value, falsify_value)
             
         Raises:
-            RuntimeError: If Z3 model not available
+            ModelStateError: If Z3 model not available
         """
         if not self.z3_model:
-            raise RuntimeError("Cannot extract state without Z3 model")
+            from .errors import ModelStateError
+            raise ModelStateError(
+                "Cannot extract verify/falsify state: No Z3 model available. "
+                "Ensure the model has been solved and is satisfiable before extraction."
+            )
         
         import z3
         
