@@ -49,7 +49,7 @@ class NegationOperator(syntactic.Operator):
         - Involutive: ¬¬A ≡ A
         - Preserves excluded middle: A ∨ ¬A is a tautology
         - Preserves non-contradiction: ¬(A ∧ ¬A) is a tautology
-        - Truth-functional: Value depends only on truth value of argument
+        - Extensional: Value depends only on truth value of argument
         
     Example:
         If p means "it's raining", then ¬p means "it's not raining"
@@ -58,33 +58,36 @@ class NegationOperator(syntactic.Operator):
     name = "\\neg"
     arity = 1
 
-    def true_at(self, argument, eval_world, eval_time):
+    def true_at(self, argument, eval_point):
         """Returns true if argument is false.
         
         Args:
             argument: The argument to negate
-            eval_world: The world ID for evaluation context
-            eval_time: The time for evaluation context
+            eval_point: Dictionary containing evaluation parameters:
+                - "world": The world ID for evaluation context
+                - "time": The time for evaluation context
         """
-        return self.semantics.false_at(argument, eval_world, eval_time)
+        return self.semantics.false_at(argument, eval_point)
 
-    def false_at(self, argument, eval_world, eval_time):
+    def false_at(self, argument, eval_point):
         """Returns false if argument is true.
         
         Args:
             argument: The argument to negate
-            eval_world: The world ID for evaluation context
-            eval_time: The time for evaluation context
+            eval_point: Dictionary containing evaluation parameters:
+                - "world": The world ID for evaluation context
+                - "time": The time for evaluation context
         """
-        return self.semantics.true_at(argument, eval_world, eval_time)
+        return self.semantics.true_at(argument, eval_point)
 
-    def find_truth_condition(self, argument, eval_world, eval_time):
+    def find_truth_condition(self, argument, eval_point):
         """Gets truth-condition for the negation of an argument.
         
         Args:
             argument: The argument to negate
-            eval_world: The world ID for evaluation context
-            eval_time: The time for evaluation context
+            eval_point: Dictionary containing evaluation parameters:
+                - "world": The world ID for evaluation context
+                - "time": The time for evaluation context
             
         Returns:
             dict: A dictionary mapping world_ids to (true_times, false_times) pairs,
@@ -113,7 +116,7 @@ class AndOperator(syntactic.Operator):
         - Identity: A ∧ ⊤ ≡ A
         - Annihilator: A ∧ ⊥ ≡ ⊥
         - Idempotent: A ∧ A ≡ A
-        - Truth-functional: Value depends only on truth values of arguments
+        - Extensional: Value depends only on truth values of arguments
         
     Example:
         If p means "it's raining" and q means "it's cold", then (p ∧ q) means 
@@ -123,44 +126,47 @@ class AndOperator(syntactic.Operator):
     name = "\\wedge"
     arity = 2
 
-    def true_at(self, leftarg, rightarg, eval_world, eval_time):
+    def true_at(self, leftarg, rightarg, eval_point):
         """Returns true if both arguments are true.
         
         Args:
             leftarg: The left argument of the conjunction
             rightarg: The right argument of the conjunction
-            eval_world: The world ID for evaluation context
-            eval_time: The time for evaluation context
+            eval_point: Dictionary containing evaluation parameters:
+                - "world": The world ID for evaluation context
+                - "time": The time for evaluation context
         """
         semantics = self.semantics
         return z3.And(
-            semantics.true_at(leftarg, eval_world, eval_time),
-            semantics.true_at(rightarg, eval_world, eval_time)
+            semantics.true_at(leftarg, eval_point),
+            semantics.true_at(rightarg, eval_point)
         )
 
-    def false_at(self, leftarg, rightarg, eval_world, eval_time):
+    def false_at(self, leftarg, rightarg, eval_point):
         """Returns true if either argument is false.
         
         Args:
             leftarg: The left argument of the conjunction
             rightarg: The right argument of the conjunction
-            eval_world: The world ID for evaluation context
-            eval_time: The time for evaluation context
+            eval_point: Dictionary containing evaluation parameters:
+                - "world": The world ID for evaluation context
+                - "time": The time for evaluation context
         """
         semantics = self.semantics
         return z3.Or(
-            semantics.false_at(leftarg, eval_world, eval_time),
-            semantics.false_at(rightarg, eval_world, eval_time)
+            semantics.false_at(leftarg, eval_point),
+            semantics.false_at(rightarg, eval_point)
         )
 
-    def find_truth_condition(self, leftarg, rightarg, eval_world, eval_time):
+    def find_truth_condition(self, leftarg, rightarg, eval_point):
         """Gets truth-condition for the conjunction of two arguments.
         
         Args:
             leftarg: The left argument of the conjunction
             rightarg: The right argument of the conjunction
-            eval_world: The world ID for evaluation context
-            eval_time: The time for evaluation context
+            eval_point: Dictionary containing evaluation parameters:
+                - "world": The world ID for evaluation context
+                - "time": The time for evaluation context
             
         Returns:
             dict: A dictionary mapping world_ids to (true_times, false_times) pairs,
@@ -202,7 +208,7 @@ class OrOperator(syntactic.Operator):
         - Identity: A ∨ ⊥ ≡ A
         - Annihilator: A ∨ ⊤ ≡ ⊤
         - Idempotent: A ∨ A ≡ A
-        - Truth-functional: Value depends only on truth values of arguments
+        - Extensional: Value depends only on truth values of arguments
         
     Example:
         If p means "it's raining" and q means "it's cold", then (p ∨ q) means 
@@ -212,44 +218,47 @@ class OrOperator(syntactic.Operator):
     name = "\\vee"
     arity = 2
 
-    def true_at(self, leftarg, rightarg, eval_world, eval_time):
+    def true_at(self, leftarg, rightarg, eval_point):
         """Returns true if either argument is true.
         
         Args:
             leftarg: The left argument of the disjunction
             rightarg: The right argument of the disjunction
-            eval_world: The world ID for evaluation context
-            eval_time: The time for evaluation context
+            eval_point: Dictionary containing evaluation parameters:
+                - "world": The world ID for evaluation context
+                - "time": The time for evaluation context
         """
         semantics = self.semantics
         return z3.Or(
-            semantics.true_at(leftarg, eval_world, eval_time),
-            semantics.true_at(rightarg, eval_world, eval_time)
+            semantics.true_at(leftarg, eval_point),
+            semantics.true_at(rightarg, eval_point)
         )
 
-    def false_at(self, leftarg, rightarg, eval_world, eval_time):
+    def false_at(self, leftarg, rightarg, eval_point):
         """Returns true if both arguments are false.
         
         Args:
             leftarg: The left argument of the disjunction
             rightarg: The right argument of the disjunction
-            eval_world: The world ID for evaluation context
-            eval_time: The time for evaluation context
+            eval_point: Dictionary containing evaluation parameters:
+                - "world": The world ID for evaluation context
+                - "time": The time for evaluation context
         """
         semantics = self.semantics
         return z3.And(
-            semantics.false_at(leftarg, eval_world, eval_time),
-            semantics.false_at(rightarg, eval_world, eval_time)
+            semantics.false_at(leftarg, eval_point),
+            semantics.false_at(rightarg, eval_point)
         )
 
-    def find_truth_condition(self, leftarg, rightarg, eval_world, eval_time):
+    def find_truth_condition(self, leftarg, rightarg, eval_point):
         """Gets truth-condition for the disjunction of two arguments.
         
         Args:
             leftarg: The left argument of the disjunction
             rightarg: The right argument of the disjunction
-            eval_world: The world ID for evaluation context
-            eval_time: The time for evaluation context
+            eval_point: Dictionary containing evaluation parameters:
+                - "world": The world ID for evaluation context
+                - "time": The time for evaluation context
             
         Returns:
             dict: A dictionary mapping world_ids to (true_times, false_times) pairs,
@@ -302,26 +311,33 @@ class BotOperator(syntactic.Operator):
     name = "\\bot"
     arity = 0
 
-    def true_at(self, eval_world, eval_time):
+    def true_at(self, eval_point):
         """Returns true if world state != itself (always false)."""
+        # Extract world and time from eval_point
+        eval_world = eval_point["world"]
+        eval_time = eval_point["time"]
         # Get the world array from the world ID
         world_array = self.semantics.world_function(eval_world)
         world_state = z3.Select(world_array, eval_time)
         return world_state != world_state
 
-    def false_at(self, eval_world, eval_time):
+    def false_at(self, eval_point):
         """Returns true if world state == itself (always true)."""
+        # Extract world and time from eval_point
+        eval_world = eval_point["world"]
+        eval_time = eval_point["time"]
         # Get the world array from the world ID
         world_array = self.semantics.world_function(eval_world)
         world_state = z3.Select(world_array, eval_time)
         return world_state == world_state
 
-    def find_truth_condition(self, eval_world, eval_time):
+    def find_truth_condition(self, eval_point):
         """Returns the extension where all times are false at all worlds.
         
         Args:
-            eval_world: The world ID for evaluation context
-            eval_time: The time for evaluation context
+            eval_point: Dictionary containing evaluation parameters:
+                - "world": The world ID for evaluation context
+                - "time": The time for evaluation context
             
         Returns:
             dict: A dictionary mapping world_ids to (true_times, false_times) pairs,
@@ -365,27 +381,37 @@ class NecessityOperator(syntactic.Operator):
     name = "\\Box"
     arity = 1
 
-    def true_at(self, argument, eval_world, eval_time):
+    def true_at(self, argument, eval_point):
         """Returns true if argument is true in all possible worlds at the eval_time."""
         semantics = self.semantics
         
+        # Extract time from eval_point
+        eval_time = eval_point["time"]
+        
         # The argument must be true in all worlds at the eval_time
         other_world = z3.Int('nec_true_world')
         # For any other_world
         return z3.ForAll(
             other_world,
             z3.Implies(
-                # If other_world is a valid world
-                semantics.is_world(other_world),
+                z3.And(
+                    # If other_world is a valid world
+                    semantics.is_world(other_world),
+                    # And eval_time is in other_world
+                    semantics.is_valid_time_for_world(other_world, eval_time),
+                ),
                 # Then the argument is true in other_world at the eval_time
-                semantics.true_at(argument, other_world, eval_time)
+                semantics.true_at(argument, {"world": other_world, "time": eval_time})
             )
         )
 
-    def false_at(self, argument, eval_world, eval_time):
+    def false_at(self, argument, eval_point):
         """Returns true if argument is false in any possible worlds at the eval_time."""
         semantics = self.semantics
         
+        # Extract time from eval_point
+        eval_time = eval_point["time"]
+        
         # The argument must be false in some world at the eval_time
         other_world = z3.Int('nec_true_world')
         # There is some other_world
@@ -394,18 +420,22 @@ class NecessityOperator(syntactic.Operator):
             z3.And(
                 # Where other_world is a valid world
                 semantics.is_world(other_world),
+                # And eval_time is in other_world
+                semantics.is_valid_time_for_world(other_world, eval_time),
                 # And the argument is false in other_world at the eval_time
-                semantics.true_at(argument, other_world, eval_time)
+                semantics.false_at(argument, {"world": other_world, "time": eval_time})
             )
         )
 
-    def find_truth_condition(self, argument, eval_world, eval_time):
+    # TODO: should this use eval_point?
+    def find_truth_condition(self, argument, eval_point):
         """Gets truth-condition for: 'It is necessary that: argument'.
         
         Args:
             argument: The argument to apply necessity to
-            eval_world: The world ID for evaluation context (not used)
-            eval_time: The time for evaluation context (not used)
+            eval_point: Dictionary containing evaluation parameters:
+                - "world": The world ID for evaluation context (not used)
+                - "time": The time for evaluation context (not used)
             
         Returns:
             dict: A dictionary mapping world_ids to (true_times, false_times) pairs.
@@ -425,6 +455,9 @@ class NecessityOperator(syntactic.Operator):
             start_time, end_time = semantics.world_time_intervals[current_world]
             # Generate a list of all valid times for this world
             world_time_interval = list(range(start_time, end_time + 1))
+            # # Skip worlds that do not include the eval_time
+            # if eval_time not in world_time_interval:
+            #     continue
             # Initialize lists to store times when necessity is true/false
             true_times, false_times = [], []
 
@@ -493,9 +526,14 @@ class FutureOperator(syntactic.Operator):
     name = "\\Future"
     arity = 1
 
-    def true_at(self, argument, eval_world, eval_time):
+    def true_at(self, argument, eval_point):
         """Returns true if argument is true at all future times in this world's interval."""
         semantics = self.semantics
+        
+        # Extract world and time from eval_point
+        eval_world = eval_point["world"]
+        eval_time = eval_point["time"]
+        
         future_time = z3.Int('future_true_time')
         return z3.ForAll(
             future_time,
@@ -506,19 +544,26 @@ class FutureOperator(syntactic.Operator):
                     # Time is in the future of eval_time
                     eval_time < future_time,
                 ),
-                semantics.true_at(argument, eval_world, future_time),
+                # Then the argument is true in the eval_world at the future_time
+                semantics.true_at(argument, {"world": eval_world, "time": future_time}),
             )
         )
     
-    def false_at(self, argument, eval_world, eval_time):
+    def false_at(self, argument, eval_point):
         """Returns true if argument is false at at least one future time in this world's interval.
         
         Args:
             argument: The argument to apply the future operator to
-            eval_world: The world ID for evaluation context
-            eval_time: The time for evaluation context
+            eval_point: Dictionary containing evaluation parameters:
+                - "world": The world ID for evaluation context
+                - "time": The time for evaluation context
         """
         semantics = self.semantics
+        
+        # Extract world and time from eval_point
+        eval_world = eval_point["world"]
+        eval_time = eval_point["time"]
+        
         future_time = z3.Int('future_false_time')
         return z3.Exists(
             future_time,
@@ -527,17 +572,19 @@ class FutureOperator(syntactic.Operator):
                 semantics.is_valid_time_for_world(eval_world, future_time),
                 # Time is in the future of eval_time
                 eval_time < future_time,
-                semantics.false_at(argument, eval_world, future_time),
+                # And the argument is true in the eval_world at the future_time
+                semantics.false_at(argument, {"world": eval_world, "time": future_time}),
             )
         )
     
-    def find_truth_condition(self, argument, eval_world, eval_time):
+    def find_truth_condition(self, argument, eval_point):
         """Gets truth-condition for 'It will always be the case that: argument'.
         
         Args:
             argument: The argument to apply the future operator to
-            eval_world: The world ID for evaluation context
-            eval_time: The time for evaluation context
+            eval_point: Dictionary containing evaluation parameters:
+                - "world": The world ID for evaluation context
+                - "time": The time for evaluation context
             
         Returns:
             dict: A dictionary mapping world_ids to (true_times, false_times) pairs,
@@ -599,8 +646,10 @@ class FutureOperator(syntactic.Operator):
     
     def print_method(self, sentence_obj, eval_point, indent_num, use_colors):
         """Print temporal operator evaluation across different time points."""
-        all_times = sentence_obj.proposition.model_structure.all_times
-        self.print_over_times(sentence_obj, eval_point, all_times, indent_num, use_colors)
+        eval_world = eval_point["world"]
+        eval_world_history = sentence_obj.proposition.model_structure.get_world_history(eval_world)
+        eval_world_times = eval_world_history.keys()
+        self.print_over_times(sentence_obj, eval_point, eval_world_times, indent_num, use_colors)
 
 
 class PastOperator(syntactic.Operator):
@@ -630,57 +679,72 @@ class PastOperator(syntactic.Operator):
     name = "\\Past"
     arity = 1
 
-    def true_at(self, argument, eval_world, eval_time):
+    def true_at(self, argument, eval_point):
         """Returns true if argument is true at all past times in this world's interval.
         
         Args:
             argument: The argument to apply the past operator to
-            eval_world: The world ID for evaluation context
-            eval_time: The time for evaluation context
+            eval_point: Dictionary containing evaluation parameters:
+                - "world": The world ID for evaluation context
+                - "time": The time for evaluation context
         """
         semantics = self.semantics
-        time = z3.Int('past_true_time')
+        
+        # Extract world and time from eval_point
+        eval_world = eval_point["world"]
+        eval_time = eval_point["time"]
+        
+        past_time = z3.Int('past_true_time')
         return z3.ForAll(
-            time,
+            past_time,
             z3.Implies(
                 z3.And(
-                    # Time is within the valid range for this world's interval
-                    semantics.is_valid_time_for_world(eval_world, time),
-                    # Time is in the past of eval_time
-                    eval_time > time,
+                    # If the past_time is within the eval_world's time interval
+                    semantics.is_valid_time_for_world(eval_world, past_time),
+                    # And the past_time is before the eval_time
+                    past_time < eval_time,
                 ),
-                semantics.true_at(argument, eval_world, time),
+                # Then the argument is true at the past_time in the eval_world
+                semantics.true_at(argument, {"world": eval_world, "time": past_time}),
             )
         )
     
-    def false_at(self, argument, eval_world, eval_time):
+    def false_at(self, argument, eval_point):
         """Returns true if argument is false at at least one past time in this world's interval.
         
         Args:
             argument: The argument to apply the past operator to
-            eval_world: The world ID for evaluation context
-            eval_time: The time for evaluation context
+            eval_point: Dictionary containing evaluation parameters:
+                - "world": The world ID for evaluation context
+                - "time": The time for evaluation context
         """
         semantics = self.semantics
-        time = z3.Int('past_false_time')
+        
+        # Extract world and time from eval_point
+        eval_world = eval_point["world"]
+        eval_time = eval_point["time"]
+        
+        past_time = z3.Int('past_false_time')
         return z3.Exists(
-            time,
+            past_time,
             z3.And(
-                # Time is within the valid range for this world's interval
-                semantics.is_valid_time_for_world(eval_world, time),
-                # Time is in the past of eval_time
-                eval_time > time,
-                semantics.false_at(argument, eval_world, time),
+                # The past_time is within the eval_world's time interval
+                semantics.is_valid_time_for_world(eval_world, past_time),
+                # The past_time is before the eval_time
+                past_time < eval_time,
+                # And the argument is true at the past_time in the eval_world
+                semantics.false_at(argument, {"world": eval_world, "time": past_time}),
             )
         )
     
-    def find_truth_condition(self, argument, eval_world, eval_time):
+    def find_truth_condition(self, argument, eval_point):
         """Gets truth-condition for 'It has always been the case that: argument'.
         
         Args:
             argument: The argument to apply the past operator to
-            eval_world: The world ID for evaluation context
-            eval_time: The time for evaluation context
+            eval_point: Dictionary containing evaluation parameters:
+                - "world": The world ID for evaluation context
+                - "time": The time for evaluation context
             
         Returns:
             dict: A dictionary mapping world_ids to (true_times, false_times) pairs,
@@ -749,8 +813,12 @@ class PastOperator(syntactic.Operator):
             indent_num: The indentation level
             use_colors: Whether to use colors in output
         """
-        all_times = sentence_obj.proposition.model_structure.all_times
-        self.print_over_times(sentence_obj, eval_point, all_times, indent_num, use_colors)
+        eval_world = eval_point["world"]
+        eval_world_history = sentence_obj.proposition.model_structure.get_world_history(eval_world)
+        eval_world_times = eval_world_history.keys()
+        self.print_over_times(sentence_obj, eval_point, eval_world_times, indent_num, use_colors)
+
+
 
 
 
@@ -767,7 +835,7 @@ class ConditionalOperator(syntactic.DefinedOperator):
     Key Properties:
         - Defined as: A → B ≡ ¬A ∨ B
         - Vacuously true when antecedent is false
-        - Truth-functional: Value depends only on truth values of arguments
+        - Extensional: Value depends only on truth values of arguments
         - Not equivalent to natural language "if-then"
         - Supports modus ponens: From A and A → B, infer B
         - Supports modus tollens: From ¬B and A → B, infer ¬A
@@ -799,7 +867,7 @@ class BiconditionalOperator(syntactic.DefinedOperator):
         - Defined as: A ↔ B ≡ (A → B) ∧ (B → A)
         - Commutative: A ↔ B ≡ B ↔ A
         - Reflexive: A ↔ A is a tautology
-        - Truth-functional: Value depends only on truth values of arguments
+        - Extensional: Value depends only on truth values of arguments
         - Not equivalent to natural language "if and only if"
         - Represents logical equivalence between formulas
         
@@ -937,8 +1005,11 @@ class DefFutureOperator(syntactic.DefinedOperator):
     
     def print_method(self, sentence_obj, eval_point, indent_num, use_colors):
         """Print temporal operator evaluation across different time points."""
-        all_times = sentence_obj.proposition.model_structure.all_times
-        self.print_over_times(sentence_obj, eval_point, all_times, indent_num, use_colors)
+        eval_world = eval_point["world"]
+        eval_world_history = sentence_obj.proposition.model_structure.get_world_history(eval_world)
+        eval_world_times = eval_world_history.keys()
+        self.print_over_times(sentence_obj, eval_point, eval_world_times, indent_num, use_colors)
+
 
 class DefPastOperator(syntactic.DefinedOperator):
 
@@ -950,8 +1021,12 @@ class DefPastOperator(syntactic.DefinedOperator):
     
     def print_method(self, sentence_obj, eval_point, indent_num, use_colors):
         """Print temporal operator evaluation across different time points."""
-        all_times = sentence_obj.proposition.model_structure.all_times
-        self.print_over_times(sentence_obj, eval_point, all_times, indent_num, use_colors)
+        eval_world = eval_point["world"]
+        eval_world_history = sentence_obj.proposition.model_structure.get_world_history(eval_world)
+        eval_world_times = eval_world_history.keys()
+        self.print_over_times(sentence_obj, eval_point, eval_world_times, indent_num, use_colors)
+
+
 
 
 bimodal_operators = syntactic.OperatorCollection(
