@@ -49,7 +49,6 @@ from .semantic import ImpositionSemantics
 from .operators import imposition_operators  # Use the OperatorCollection
 from . import Proposition, ModelStructure  # Now imported from logos via __init__.py
 
-# Import logos theory components for comparison
 from model_checker.theory_lib.logos import (
     LogosSemantics,
     LogosProposition,
@@ -62,6 +61,27 @@ from model_checker.theory_lib.logos import (
 ### COUNTERMODELS ###
 #####################
 
+# IM_TR_0: TEST DERIVED IMPOSITION CONSTRAINTS
+# This tests whether all derived constraints from is_alternative are entailed
+# by the base semantics. If no model is found, all constraints are entailed.
+IM_TR_0_premises = []
+IM_TR_0_conclusions = []
+IM_TR_0_settings = {
+    'N' : 4,
+    'contingent' : False,
+    'non_null' : False,
+    'non_empty' : False,
+    'disjoint' : False,
+    'max_time' : 10,
+    'iterate' : 1,
+    'expectation' : True,
+}
+IM_TR_0_example = [
+    IM_TR_0_premises,
+    IM_TR_0_conclusions,
+    IM_TR_0_settings,
+]
+
 # IM_CM_0: COUNTERFACTUAL ANTECEDENT STRENGTHENING
 IM_CM_0_premises = ['\\neg A', '(A \\diamondright C)', '(A \\boxright C)']
 IM_CM_0_conclusions = ['((A \\wedge B) \\boxright C)']
@@ -72,7 +92,7 @@ IM_CM_0_settings = {
     'non_empty' : True,
     'disjoint' : False,
     'max_time' : 10,
-    'iterate' : 1,
+    'iterate' : 2,
     'expectation' : True,
 }
 IM_CM_0_example = [
@@ -180,7 +200,7 @@ IM_CM_5_example = [
 IM_CM_6_premises = ['\\neg A', '(A \\boxright B)','(A \\boxright C)']
 IM_CM_6_conclusions = ['((A \\wedge B) \\boxright C)']
 IM_CM_6_settings = {
-    'N' : 4,
+    'N' : 3,
     'contingent' : True,
     'non_null' : True,
     'non_empty' : True,
@@ -256,7 +276,7 @@ IM_CM_9_example = [
 IM_CM_10_premises = ['(A \\boxright B)','(B \\boxright C)']
 IM_CM_10_conclusions = ['(A \\boxright C)']
 IM_CM_10_settings = {
-    'N' : 4,
+    'N' : 3,
     'contingent' : True,
     'non_null' : True,
     'non_empty' : True,
@@ -657,6 +677,7 @@ IM_CM_29_example = [
 
 
 
+
 ##################
 ### THEOREMS  ###
 ##################
@@ -876,8 +897,8 @@ IM_TH_11_example = [
 
 
 
-# Create collections for different imposition example types
-imposition_cm_examples = {
+# Organize examples by category
+countermodel_examples = {
     "IM_CM_0": IM_CM_0_example,   # COUNTERFACTUAL AND MIGHT COUNTERFACTUAL ANTECEDENT STRENGTHENING
     "IM_CM_1": IM_CM_1_example,   # COUNTERFACTUAL ANTECEDENT STRENGTHENING
     "IM_CM_2": IM_CM_2_example,   # MIGHT COUNTERFACTUAL ANTECEDENT STRENGTHENING
@@ -909,7 +930,7 @@ imposition_cm_examples = {
     "IM_CM_28": IM_CM_28_example, # REVERSE FLIP DEFINITION OF NEC
 }
 
-imposition_th_examples = {
+theorem_examples = {
     "IM_TH_1": IM_TH_1_example,   # COUNTERFACTUAL IDENTITY
     "IM_TH_2": IM_TH_2_example,   # COUNTERFACTUAL MODUS PONENS
     "IM_TH_3": IM_TH_3_example,   # WEAKENED TRANSITIVITY
@@ -923,8 +944,8 @@ imposition_th_examples = {
     "IM_TH_11": IM_TH_11_example, # DEFINITION OF NEC
 }
 
-# Combined collection of all imposition examples
-unit_tests = {**imposition_cm_examples, **imposition_th_examples}
+# Combine for unit_tests (used by test framework)
+unit_tests = {**countermodel_examples, **theorem_examples}
 
 # Aliases for main dictionary (backward compatibility)
 test_example_range = unit_tests
@@ -937,6 +958,7 @@ general_settings = {
     "print_z3": False,
     "save_output": False,
     "maximize": False,
+    "derive_imposition": False,
 }
 
 # Create operator registry for logos theory with counterfactual operators
@@ -975,33 +997,20 @@ semantic_theories = {
 # Default example range (curated subset for direct execution)
 example_range = {
 
-    # # DEMO 1
-    "IM_CM_0": IM_CM_0_example,   # COUNTERFACTUAL AND MIGHT COUNTERFACTUAL ANTECEDENT STRENGTHENING
-    # "IM_TH_5": IM_TH_5_example,   # SIMPLIFICATION OF DISJUNCTIVE ANTECEDENT
-
-    # RUN UNIT_TEST: ./Code/run_tests.py --examples logos counterfactuals
-    # RUN UNIT_TEST: ./Code/run_tests.py --examples imposition
-
-    # # DEMO 2
-    # RUN MAXIMIZER: ./Code/dev_cli.py /home/benjamin/Documents/Philosophy/Projects/ModelChecker/Code/src/model_checker/theory_lib/imposition/examples.py -m
-
-    # # DEMO 3
-    # "IM_CM_1": IM_CM_1_example,   # COUNTERFACTUAL AND MIGHT COUNTERFACTUAL ANTECEDENT STRENGTHENING
-    # "IM_CM_22": IM_CM_22_example, # REVERSE DEFINITION OF NEC
-    # "IM_TH_11": IM_TH_11_example, # DEFINITION OF NEC
-
-    # # FLIP DEFINITION
-    # "IM_CM_28": IM_CM_28_example, # REVERSE FLIP DEFINITION OF NEC
-    # "IM_CM_29": IM_CM_29_example, # FLIP DEFINITION OF NEC
-
-    # # EXTRA: cannot be run with logos_theory
-    # "IM_CM_23": IM_CM_23_example, # CONTRADICTION TO IMPOSSIBILITY
-    # "IM_CM_26": IM_CM_26_example, # IMPOSITION TO LOGOS COUNTERFACTUAL
-    # "IM_CM_27": IM_CM_27_example, # LOGOS TO IMPOSITION COUNTERFACTUAL
-
     # Countermodels
-    # "IM_CM_0": IM_CM_0_example,   # COUNTERFACTUAL AND MIGHT COUNTERFACTUAL ANTECEDENT STRENGTHENING
-    "IM_CM_1": IM_CM_1_example,   # COUNTERFACTUAL ANTECEDENT STRENGTHENING
+
+    # "IM_TH_5": IM_TH_5_example,   # SIMPLIFICATION OF DISJUNCTIVE ANTECEDENT
+    # "IM_TR_0": IM_TR_0_example,   # NO PREMISES OR CONCLUSIONS
+
+    "IM_CM_0": IM_CM_0_example,   # COUNTERFACTUAL AND MIGHT COUNTERFACTUAL ANTECEDENT STRENGTHENING
+    # "IM_CM_1": IM_CM_1_example,   # COUNTERFACTUAL ANTECEDENT STRENGTHENING
+    #
+    # "IM_TH_11": IM_TH_11_example, # DEFINITION OF NEC
+    # "IM_CM_22": IM_CM_22_example, # REVERSE DEFINITION OF NEC
+    #
+    # "IM_CM_29": IM_CM_29_example, # FLIP DEFINITION OF NEC
+    # "IM_CM_28": IM_CM_28_example, # REVERSE FLIP DEFINITION OF NEC
+
     # "IM_CM_2": IM_CM_2_example,   # MIGHT COUNTERFACTUAL ANTECEDENT STRENGTHENING
     # "IM_CM_3": IM_CM_3_example,   # COUNTERFACTUAL ANTECEDENT STRENGTHENING WITH POSSIBILITY
     # "IM_CM_4": IM_CM_4_example,   # COUNTERFACTUAL ANTECEDENT STRENGTHENING WITH NEGATION
@@ -1022,29 +1031,30 @@ example_range = {
     # "IM_CM_19": IM_CM_19_example, # COUNTERFACTUAL EXPORTATION
     # "IM_CM_20": IM_CM_20_example, # COUNTERFACTUAL EXPORTATION WITH POSSIBILITY
     # "IM_CM_21": IM_CM_21_example, # COUNTERFACTUAL NEGATION DISTRIBUTION
-    # "IM_CM_22": IM_CM_22_example, # REVERSE DEFINITION OF NEC
+
+    # "IM_CM_26": IM_CM_26_example, # IMPOSITION TO LOGOS COUNTERFACTUAL
+    # "IM_CM_27": IM_CM_27_example, # LOGOS TO IMPOSITION COUNTERFACTUAL
+
     # "IM_CM_23": IM_CM_23_example, # CONTRADICTION TO IMPOSSIBILITY
     # "IM_CM_24": IM_CM_24_example, # NECESSITY OF COUNTERFACTUALS
     # "IM_CM_25": IM_CM_25_example, # INCOMPATIBILITY TO COUNTERFACTUAL
-    # "IM_CM_26": IM_CM_26_example, # IMPOSITION TO LOGOS COUNTERFACTUAL
-    # "IM_CM_27": IM_CM_27_example, # LOGOS TO IMPOSITION COUNTERFACTUAL
+
 
     # # Theorems
     # "IM_TH_1": IM_TH_1_example,   # COUNTERFACTUAL IDENTITY
     # "IM_TH_2": IM_TH_2_example,   # COUNTERFACTUAL MODUS PONENS
     # "IM_TH_3": IM_TH_3_example,   # WEAKENED TRANSITIVITY
     # "IM_TH_4": IM_TH_4_example,   # ANTECEDENT DISJUNCTION TO CONJUNCTION
-    # "IM_TH_5": IM_TH_5_example,   # SIMPLIFICATION OF DISJUNCTIVE ANTECEDENT
     # "IM_TH_6": IM_TH_6_example,   # DOUBLE SIMPLIFICATION OF DISJUNCTIVE ANTECEDENT
     # "IM_TH_7": IM_TH_7_example,   # COUNTERFACTUAL DISJUNCTION INTRODUCTION
     # "IM_TH_8": IM_TH_8_example,   # COUNTERFACTUAL CONSEQUENT WEAKENING
     # "IM_TH_9": IM_TH_9_example,   # COUNTERFACTUAL CONJUNCTION INTRODUCTION
     # "IM_TH_10": IM_TH_10_example, # MIGHT FACTIVITY
-    # "IM_TH_11": IM_TH_11_example, # DEFINITION OF NEC
     
 }
 
 
+
 def get_examples():
     """
     Get all imposition examples.
