@@ -9,11 +9,9 @@ import z3
 import sys
 import time
 
-from model_checker.model import (
-    PropositionDefaults,
-    SemanticDefaults,
-    ModelDefaults,
-)
+from model_checker.models.proposition import PropositionDefaults
+from model_checker.models.semantic import SemanticDefaults
+from model_checker.models.structure import ModelDefaults
 from model_checker.utils import (
     ForAll,
     Exists,
@@ -40,8 +38,6 @@ class LogosSemantics(SemanticDefaults):
         'disjoint': True,
         'max_time': 10,
         'iterate': False,
-        'iteration_timeout': 1.0,
-        'iteration_attempts': 5,
         'expectation': None,
     }
     
@@ -388,6 +384,65 @@ class LogosSemantics(SemanticDefaults):
         # Placeholder: for now, just return False (no ordering)
         # A real implementation would define closeness based on similarity metrics
         return z3.BoolVal(False)
+    
+    def inject_z3_model_values(self, z3_model, original_semantics, model_constraints):
+        """Inject concrete Z3 values from iteration into model constraints.
+        
+        This method extracts values from a Z3 model and adds them as constraints
+        for the next iteration. It handles Logos-specific concepts: worlds,
+        possible states, verify, and falsify relations.
+        
+        Args:
+            z3_model: Z3 model containing concrete values from solver
+            original_semantics: Original semantics instance that created the Z3 functions
+            model_constraints: ModelConstraints instance to update with injected values
+        """
+        # Get number of states from model_constraints settings
+        num_states = 2 ** model_constraints.settings['N']
+        
+        # Inject world constraints
+        for state in range(num_states):
+            # Evaluate using original is_world function
+            is_world_val = z3_model.eval(original_semantics.is_world(state), model_completion=True)
+            # Add constraint using new is_world function
+            if z3.is_true(is_world_val):
+                model_constraints.all_constraints.append(self.is_world(state))
+            else:
+                model_constraints.all_constraints.append(z3.Not(self.is_world(state)))
+        
+        # Inject possible state constraints
+        for state in range(num_states):
+            # Evaluate using original possible function
+            is_possible_val = z3_model.eval(original_semantics.possible(state), model_completion=True)
+            # Add constraint using new possible function
+            if z3.is_true(is_possible_val):
+                model_constraints.all_constraints.append(self.possible(state))
+            else:
+                model_constraints.all_constraints.append(z3.Not(self.possible(state)))
+        
+        # Inject verify/falsify constraints for each sentence letter
+        for sentence_obj in model_constraints.syntax.sentence_letters:
+            atom = sentence_obj.sentence_letter
+            
+            # Inject verify constraints
+            for state in range(num_states):
+                # Evaluate using original verify function
+                verify_val = z3_model.eval(original_semantics.verify(state, atom), model_completion=True)
+                # Add constraint using new verify function
+                if z3.is_true(verify_val):
+                    model_constraints.all_constraints.append(self.verify(state, atom))
+                else:
+                    model_constraints.all_constraints.append(z3.Not(self.verify(state, atom)))
+            
+            # Inject falsify constraints
+            for state in range(num_states):
+                # Evaluate using original falsify function
+                falsify_val = z3_model.eval(original_semantics.falsify(state, atom), model_completion=True)
+                # Add constraint using new falsify function
+                if z3.is_true(falsify_val):
+                    model_constraints.all_constraints.append(self.falsify(state, atom))
+                else:
+                    model_constraints.all_constraints.append(z3.Not(self.falsify(state, atom)))
 
 
 class LogosProposition(PropositionDefaults):
@@ -573,6 +628,13 @@ class LogosProposition(PropositionDefaults):
                  states that verify and falsify the proposition respectively
         """
         model = self.model_structure.z3_model
+        if model is None:
+            # If no model is available, return empty sets
+            # This can happen during iteration when models are being created
+            return set(), set()
+        
+        # Clean proposition computation without debug output
+            
         semantics = self.semantics
         eval_world = self.eval_world
         operator = self.operator
@@ -581,17 +643,61 @@ class LogosProposition(PropositionDefaults):
         if sentence_letter is not None:
             V = {
                 state for state in self.model_structure.all_states
-                if model.evaluate(semantics.verify(state, sentence_letter))
+                if self._evaluate_z3_boolean(model, semantics.verify(state, sentence_letter))
             }
             F = {
                 state for state in self.model_structure.all_states
-                if model.evaluate(semantics.falsify(state, sentence_letter))
+                if self._evaluate_z3_boolean(model, semantics.falsify(state, sentence_letter))
             }
             return V, F
         if operator is not None:
             eval_point = {"world": eval_world}
             return operator.find_verifiers_and_falsifiers(*arguments, eval_point)
         raise ValueError(f"Their is no proposition for {self}.")
+    
+    def _evaluate_z3_boolean(self, z3_model, expression):
+        """Safely evaluate a Z3 boolean expression to a Python boolean.
+        
+        This method handles the case where Z3 returns symbolic expressions
+        instead of concrete boolean values.
+        
+        Args:
+            z3_model: The Z3 model to use for evaluation
+            expression: The Z3 boolean expression to evaluate
+            
+        Returns:
+            bool: True if the expression evaluates to true, False otherwise
+        """
+        import z3
+        
+        try:
+            # Evaluate the expression with model completion
+            result = z3_model.evaluate(expression, model_completion=True)
+            
+            # Check if result is a boolean constant
+            if z3.is_true(result):
+                return True
+            elif z3.is_false(result):
+                return False
+            
+            # Try to simplify
+            simplified = z3.simplify(result)
+            if z3.is_true(simplified):
+                return True
+            elif z3.is_false(simplified):
+                return False
+                
+            # Check string representation as last resort
+            if str(simplified) == "True":
+                return True
+            elif str(simplified) == "False":
+                return False
+                
+            # Conservative default
+            return False
+            
+        except Exception:
+            return False
 
     def truth_value_at(self, eval_world):
         """Determines the truth value of the proposition at a given world.
@@ -618,12 +724,12 @@ class LogosProposition(PropositionDefaults):
         exists_verifier = False
         exists_falsifier = False
         for verifier in self.verifiers:
-            if z3_model.evaluate(semantics.is_part_of(verifier, eval_world)):
+            if z3_model.eval(semantics.is_part_of(verifier, eval_world), model_completion=True):
                 ver_witness = verifier
                 exists_verifier = True
                 break
         for falsifier in self.falsifiers:
-            if z3_model.evaluate(semantics.is_part_of(falsifier, eval_world)):
+            if z3_model.eval(semantics.is_part_of(falsifier, eval_world), model_completion=True):
                 fal_witness = falsifier
                 exists_falsifier = True
                 break
@@ -775,8 +881,9 @@ class LogosModelStructure(ModelDefaults):
             self.print_model(output)
             if output is sys.__stdout__:
                 total_time = round(time.time() - self.start_time, 4) 
-                print(f"Total Run Time: {total_time} seconds\n", file=output)
-                print(f"{'='*40}", file=output)
+                print(f"Total Run Time: {total_time} seconds", file=output)
+            # Always print closing separator for countermodels  
+            print(f"\n{'='*40}", file=output)
             return
 
     def print_to(self, default_settings, example_name, theory_name, print_constraints=None, output=sys.__stdout__):
@@ -834,50 +941,53 @@ class LogosModelStructure(ModelDefaults):
         print(f"\n{YELLOW}=== DIFFERENCES FROM PREVIOUS MODEL ==={RESET}\n", file=output)
         
         # Print world changes
-        if diffs.get('worlds', {}).get('added') or diffs.get('worlds', {}).get('removed'):
+        if diffs.get('world_changes', {}).get('added') or diffs.get('world_changes', {}).get('removed'):
             print(f"{BLUE}World Changes:{RESET}", file=output)
-            for world in diffs['worlds'].get('added', []):
+            for world in diffs.get('world_changes', {}).get('added', []):
                 world_str = bitvec_to_substates(world, self.N)
                 print(f"  {GREEN}+ {world_str} (now a world){RESET}", file=output)
-            for world in diffs['worlds'].get('removed', []):
+            for world in diffs.get('world_changes', {}).get('removed', []):
                 world_str = bitvec_to_substates(world, self.N)
                 print(f"  {RED}- {world_str} (no longer a world){RESET}", file=output)
             print("", file=output)
         
         # Print possible state changes
-        if diffs.get('possible_states', {}).get('added') or diffs.get('possible_states', {}).get('removed'):
+        if diffs.get('possible_changes', {}).get('added') or diffs.get('possible_changes', {}).get('removed'):
             print(f"{BLUE}Possible State Changes:{RESET}", file=output)
-            for state in diffs['possible_states'].get('added', []):
+            for state in diffs.get('possible_changes', {}).get('added', []):
                 state_str = bitvec_to_substates(state, self.N)
                 print(f"  {GREEN}+ {state_str} (now possible){RESET}", file=output)
-            for state in diffs['possible_states'].get('removed', []):
+            for state in diffs.get('possible_changes', {}).get('removed', []):
                 state_str = bitvec_to_substates(state, self.N)
                 print(f"  {RED}- {state_str} (now impossible){RESET}", file=output)
             print("", file=output)
         
-        # Print verification changes
-        if diffs.get('verify'):
-            print(f"{BLUE}Verification Changes:{RESET}", file=output)
-            for letter, state_changes in diffs['verify'].items():
-                print(f"  Letter {letter}:", file=output)
-                for state_str, change in state_changes.items():
-                    if change['new']:
-                        print(f"    {GREEN}+ {state_str} now verifies {letter}{RESET}", file=output)
-                    else:
-                        print(f"    {RED}- {state_str} no longer verifies {letter}{RESET}", file=output)
-            print("", file=output)
-        
-        # Print falsification changes
-        if diffs.get('falsify'):
-            print(f"{BLUE}Falsification Changes:{RESET}", file=output)
-            for letter, state_changes in diffs['falsify'].items():
-                print(f"  Letter {letter}:", file=output)
-                for state_str, change in state_changes.items():
-                    if change['new']:
-                        print(f"    {GREEN}+ {state_str} now falsifies {letter}{RESET}", file=output)
-                    else:
-                        print(f"    {RED}- {state_str} no longer falsifies {letter}{RESET}", file=output)
-            print("", file=output)
+        # Print atomic changes (verify/falsify)
+        if diffs.get('atomic_changes'):
+            atomic = diffs.get('atomic_changes', {})
+            # Print verification changes
+            if atomic.get('verify'):
+                print(f"{BLUE}Verification Changes:{RESET}", file=output)
+                for letter, state_changes in atomic['verify'].items():
+                    print(f"  Letter {letter}:", file=output)
+                    for state_str, change in state_changes.items():
+                        if change['new']:
+                            print(f"    {GREEN}+ {state_str} now verifies {letter}{RESET}", file=output)
+                        else:
+                            print(f"    {RED}- {state_str} no longer verifies {letter}{RESET}", file=output)
+                print("", file=output)
+            
+            # Print falsification changes
+            if atomic.get('falsify'):
+                print(f"{BLUE}Falsification Changes:{RESET}", file=output)
+                for letter, state_changes in atomic['falsify'].items():
+                    print(f"  Letter {letter}:", file=output)
+                    for state_str, change in state_changes.items():
+                        if change['new']:
+                            print(f"    {GREEN}+ {state_str} now falsifies {letter}{RESET}", file=output)
+                        else:
+                            print(f"    {RED}- {state_str} no longer falsifies {letter}{RESET}", file=output)
+                print("", file=output)
         
         # Print parthood changes
         if diffs.get('parthood'):
@@ -920,7 +1030,7 @@ class LogosModelStructure(ModelDefaults):
                 print(f"  {bin_rep} = {state}{label_str}", file=output)
         
         # Print formatted state space
-        print("State Space:", file=output)
+        print("\nState Space:", file=output)
         for bit in self.all_states:
             state = bitvec_to_substates(bit, self.N)
             bin_rep = binary_bitvector(bit)
@@ -932,3 +1042,120 @@ class LogosModelStructure(ModelDefaults):
                 format_state(bin_rep, state, self.COLORS["possible"])
             elif self.settings['print_impossible']:
                 format_state(bin_rep, state, self.COLORS["impossible"], "impossible")
+    
+    def extract_states(self):
+        """Extract categorized states for output.
+        
+        Logos distinguishes between worlds, possible states, and impossible states.
+        
+        Returns:
+            Dict with keys 'worlds', 'possible', 'impossible'
+        """
+        states = {"worlds": [], "possible": [], "impossible": []}
+        
+        if hasattr(self, 'z3_world_states') and self.z3_world_states:
+            for state in self.z3_world_states:
+                # Convert bit vector to state number
+                if hasattr(state, 'as_long'):
+                    states["worlds"].append(f"s{state.as_long()}")
+                else:
+                    states["worlds"].append(f"s{state}")
+        
+        if hasattr(self, 'z3_possible_states') and self.z3_possible_states:
+            for state in self.z3_possible_states:
+                # Only add if not already a world
+                if state not in (self.z3_world_states if hasattr(self, 'z3_world_states') else []):
+                    if hasattr(state, 'as_long'):
+                        states["possible"].append(f"s{state.as_long()}")
+                    else:
+                        states["possible"].append(f"s{state}")
+        
+        # For impossible states, we need to check all states that aren't possible
+        if hasattr(self, 'all_states') and self.all_states:
+            # Convert possible states to integers for comparison
+            possible_set = set()
+            if hasattr(self, 'z3_possible_states') and self.z3_possible_states:
+                for ps in self.z3_possible_states:
+                    if hasattr(ps, 'as_long'):
+                        possible_set.add(ps.as_long())
+                    else:
+                        possible_set.add(ps)
+            
+            for state in self.all_states:
+                # Convert state to integer for comparison
+                state_val = state.as_long() if hasattr(state, 'as_long') else state
+                
+                # Check if not possible and not null state
+                if state_val not in possible_set and state_val != 0:
+                    states["impossible"].append(f"s{state_val}")
+        
+        return states
+    
+    def extract_evaluation_world(self):
+        """Extract the main evaluation world.
+        
+        Returns:
+            State name (e.g., 's3') or None if not set
+        """
+        if hasattr(self, 'z3_main_world') and self.z3_main_world is not None:
+            if hasattr(self.z3_main_world, 'as_long'):
+                return f"s{self.z3_main_world.as_long()}"
+            else:
+                return f"s{self.z3_main_world}"
+        return None
+    
+    def extract_relations(self):
+        """Extract relations between states.
+        
+        For Logos, this includes fusion/fission relations and compatibility.
+        
+        Returns:
+            Dict containing various relations
+        """
+        relations = {}
+        
+        # Add any Logos-specific relations here
+        # For now, return empty as relations are computed dynamically
+        
+        return relations
+    
+    def extract_propositions(self):
+        """Extract proposition truth values at worlds.
+        
+        Returns:
+            Dict mapping propositions to their truth values at each world
+        """
+        propositions = {}
+        
+        if not hasattr(self, 'syntax') or not hasattr(self.syntax, 'propositions'):
+            return propositions
+        
+        # Get world states
+        worlds = []
+        if hasattr(self, 'z3_world_states'):
+            worlds = self.z3_world_states
+        
+        # Extract truth values for each proposition
+        for prop_name, prop_obj in self.syntax.propositions.items():
+            if hasattr(prop_obj, 'letter'):
+                letter = prop_obj.letter
+                propositions[letter] = {}
+                
+                for world in worlds:
+                    # Get world number
+                    if hasattr(world, 'as_long'):
+                        world_num = world.as_long()
+                    else:
+                        world_num = world
+                    
+                    world_name = f"s{world_num}"
+                    
+                    if hasattr(prop_obj, 'truth_value_at'):
+                        try:
+                            # Logos propositions use truth_value_at
+                            propositions[letter][world_name] = prop_obj.truth_value_at(world_num)
+                        except:
+                            # If evaluation fails, skip this world
+                            pass
+        
+        return propositions
