@@ -864,9 +864,48 @@ semantic_theories = {
 
 # Or set specific examples
 example_range = {
+
+    # COUNTERMODELS
     "CF_CM_1": CF_CM_1_example,
     # "CF_CM_2": CF_CM_2_example,
-    # "CF_TH_5": CF_TH_5_example,
+    # "CF_CM_3": CF_CM_3_example,
+    # "CF_CM_4": CF_CM_4_example,
+    # "CF_CM_5": CF_CM_5_example,
+    # "CF_CM_6": CF_CM_6_example,
+    "CF_CM_7": CF_CM_7_example,
+    # "CF_CM_8": CF_CM_8_example,
+    # "CF_CM_9": CF_CM_9_example,
+    # "CF_CM_10": CF_CM_10_example,
+    # "CF_CM_11": CF_CM_11_example,
+    # "CF_CM_12": CF_CM_12_example,
+    # "CF_CM_13": CF_CM_13_example,
+    # "CF_CM_14": CF_CM_14_example,
+    # "CF_CM_15": CF_CM_15_example,
+    # "CF_CM_16": CF_CM_16_example,
+    # "CF_CM_17": CF_CM_17_example,
+    # "CF_CM_18": CF_CM_18_example,
+    # "CF_CM_19": CF_CM_19_example,
+    # "CF_CM_20": CF_CM_20_example,
+    # "CF_CM_21": CF_CM_21_example,
+    # "CF_CM_24": CF_CM_24_example,
+    # "CF_CM_25": CF_CM_25_example,
+
+    # THEOREMS
+    # "CF_TH_1": CF_TH_1_example,
+    # "CF_TH_2": CF_TH_2_example,
+    # "CF_TH_3": CF_TH_3_example,
+    # "CF_TH_4": CF_TH_4_example,
+    "CF_TH_5": CF_TH_5_example,
+    # "CF_TH_6": CF_TH_6_example,
+    # "CF_TH_7": CF_TH_7_example,
+    # "CF_TH_8": CF_TH_8_example,
+    # "CF_TH_9": CF_TH_9_example,
+    "CF_TH_10": CF_TH_10_example,
+    # "CF_TH_11": CF_TH_11_example,
+    # "CF_TH_12": CF_TH_12_example,
+    # "CF_TH_13": CF_TH_13_example,
+    # "CF_TH_14": CF_TH_14_example,
+
 }
 
 
