@@ -113,7 +113,7 @@ MOD_CM_3_example = [
     MOD_CM_3_settings,
 ]
 
-# MOD_CM_3: MATERIAL IMPLICATION TO COUNTERFACTUAL
+# MOD_CM_4: MATERIAL IMPLICATION TO COUNTERFACTUAL
 MOD_CM_4_premises = ['(A \\rightarrow B)']
 MOD_CM_4_conclusions = ['(A \\boxright B)']
 MOD_CM_4_settings = {
@@ -526,6 +526,7 @@ modal_cm_examples = {
     "MOD_CM_1": MOD_CM_1_example,  # POSSIBILITY DOES NOT ENTAIL NECESSITY
     "MOD_CM_2": MOD_CM_2_example,  # POSSIBILITY TO ACTUALITY
     "MOD_CM_3": MOD_CM_3_example,  # COUNTERFACTUAL TO STRICT IMPLICATION
+    "MOD_CM_4": MOD_CM_4_example,  # MATERIAL IMPLICATION TO COUNTERFACTUAL
 }
 
 modal_th_examples = {
@@ -594,17 +595,31 @@ example_range = {
     "MOD_CM_1": MOD_CM_1_example,  # POSSIBILITY TO NECESSITY
     # "MOD_CM_2": MOD_CM_2_example,  # POSSIBILITY TO ACTUALITY
     # "MOD_CM_3": MOD_CM_3_example,  # MATERIAL IMPLICATION TO COUNTERFACTUAL
-    # "MOD_CM_4": MOD_CM_4_example,  # COUNTERFACTUAL TO STRICT IMPLICATION
+    # "MOD_CM_4": MOD_CM_4_example,  # MATERIAL IMPLICATION TO COUNTERFACTUAL
 
-    # # THEOREMS
+    # THEOREMS
+    # "MOD_TH_1": MOD_TH_1_example,   # NECESSITY DISTRIBUTION OVER CONJUNCTION
+    # "MOD_TH_2": MOD_TH_2_example,   # POSSIBILITY DISTRIBUTION OVER DISJUNCTION
+    # "MOD_TH_3": MOD_TH_3_example,   # MODAL DUALITY (Box to Diamond)
+    # "MOD_TH_4": MOD_TH_4_example,   # MODAL DUALITY (Diamond to Box)
     # "MOD_TH_5": MOD_TH_5_example,   # MODAL K AXIOM
     # "MOD_TH_6": MOD_TH_6_example,   # NECESSITATION RULE
     # "MOD_TH_7": MOD_TH_7_example,   # COUNTERFACTUAL NECESSITY IMPLIES NECESSITY
     # "MOD_TH_8": MOD_TH_8_example,   # POSSIBILITY IMPLIES COUNTERFACTUAL POSSIBILITY
+    # "MOD_TH_9": MOD_TH_9_example,   # COUNTERFACTUAL MODAL DUALITY
+    # "MOD_TH_10": MOD_TH_10_example, # DOUBLE NECESSITY
+    # "MOD_TH_11": MOD_TH_11_example, # 5 AXIOM (BOX)
+    # "MOD_TH_12": MOD_TH_12_example, # BOX-TO-TOP EQUIVALENCE
+    # "MOD_TH_13": MOD_TH_13_example, # TOP-TO-BOX EQUIVALENCE
+    # "MOD_TH_14": MOD_TH_14_example, # NECESSARY EQUIVALENCE OF TAUTOLOGIES
 
-    # # DEFINED
+    # DEFINED
+    # "MOD_DEF_1": MOD_DEF_1_example,  # PRIMITIVE VS DEFINED NECESSITY
     # "MOD_DEF_2": MOD_DEF_2_example,  # DEFINED VS PRIMITIVE NECESSITY
     # "MOD_DEF_3": MOD_DEF_3_example,  # PRIMITIVE VS DEFINED POSSIBILITY
+    # "MOD_DEF_4": MOD_DEF_4_example,  # DEFINED VS PRIMITIVE POSSIBILITY
+    # "MOD_DEF_5": MOD_DEF_5_example,  # NECESSITY AND NEGATED POSSIBILITY
+    # "MOD_DEF_6": MOD_DEF_6_example,  # POSSIBILITY AND NEGATED NECESSITY
 
 }
 
